@@ -1,3 +1,4 @@
+import inspect
 from logging import getLogger
 from time import time
 from typing import Any, Callable
@@ -19,31 +20,26 @@ def _is_valid(val):
     return val == val
 
 
-def try_custom_warmup(model, shape):
-    # Allow custom warm-up function defined in model, e.g.
-    # class Model:
-    #
-    #   def warm_up(self, input_shape: List[int]):
-    #       ...
-    success = False
-    try:
-        if hasattr(model, "warm_up"):
-            model.warm_up(shape)
-            success = True
-    except Exception:
-        pass
-
-    return success
+def get_call_arg_names(module_or_fn):
+    if isinstance(module_or_fn, torch.nn.Module):
+        return inspect.getfullargspec(module_or_fn.forward)[0][1:]
+    return inspect.getfullargspec(module_or_fn)[0]
 
 
 def measure_flops(model, sample, print_details=False):
     flops = _INVALID
+
+    def input_constructor(*args, **kwrags):
+        nonlocal sample
+        return {get_call_arg_names(model)[0]: sample}
+
     try:
         flops, _ = get_model_complexity_info(
             model,
-            tuple(sample.shape[1:]),
-            as_strings=False,
+            (1,),  # dummy
             print_per_layer_stat=print_details,
+            as_strings=False,
+            input_constructor=input_constructor,
             verbose=print_details,
         )
         flops = int(flops)
@@ -114,7 +110,7 @@ def measure_detailed_inference_timing(
 ):
 
     try:
-        with torch.no_grad(), torch.autograd.profiler.profile(
+        with torch.autograd.profiler.profile(
             use_cuda=(model_device.type == "cuda"), profile_memory=True
         ) as prof:
             transfer_to_device_fn(
@@ -146,7 +142,7 @@ def measure_repeated_inference_timing(
     t_tot = []
 
     for _ in tqdm(
-        range(num_runs), desc=f"Measuring inference with batch_size={batch_size}"
+        range(num_runs), desc=f"Measuring inference for batch_size={batch_size}"
     ):
         start_on_cpu = time()
         device_sample = transfer_to_device_fn(sample, model_device)
@@ -203,8 +199,10 @@ def measure_energy(
     sample,
     model_device,
     transfer_to_device_fn=torch.Tensor.to,
-    print_details=False,
+    num_runs=100,
+    batch_size: int = None,
     include_transfer_costs=True,
+    print_fn=logger.info,
 ):
     inference_joules = _INVALID
 
@@ -251,10 +249,14 @@ def measure_energy(
     try:
         from .jetson_power import PowerEstimator
 
-        p_est = PowerEstimator()
+        p_est = PowerEstimator(print_fn=print_fn)
         # index 0 is total energy, index 1 is energy over idle consumption:
-        total_joules = p_est.estimate_fn_power(test_fn)[0] / 1000
-        inference_joules = total_joules
+        meas = []
+        for _ in tqdm(
+            range(num_runs), desc=f"Measuring energy for batch_size={batch_size}"
+        ):
+            meas.append(p_est.estimate_fn_power(test_fn)[0] / 1000)
+        inference_joules = float(np.array(meas).mean())
     except Exception:
         pass
 
@@ -279,6 +281,8 @@ def benchmark(
     transfer_to_device_fn=torch.Tensor.to,
     sample_with_batch_size1: Any = None,
     batch_size: int = None,
+    print_fn=logger.info,
+    warm_up_fn=warm_up,
 ):
     results = {}
     batch_size = batch_size or sample.shape[0]
@@ -299,92 +303,121 @@ def benchmark(
     # Get machine info
     machine_info = get_machine_info()
     results["machine_info"] = machine_info
-    logger.info(fmt({"Machine info": machine_info}))
+    print_fn(fmt({"Machine info": machine_info}))
 
     model_device = get_device_fn(model)
     assert isinstance(
         model_device, torch.device
     ), "model_device should be a `torch.device`"
     results["device"] = model_device.type
-    logger.info(f"Model device: {model_device}")
+    print_fn(f"Model device: {model_device}")
 
     # Measure params
     params = measure_params(model)
     if _is_valid(params):
         results["params"] = params
-        logger.info(f"Model parameters: {params} ({format_num(params)})")
+        print_fn(f"Model parameters: {params} ({format_num(params)})")
 
     # Measure FLOPs
-    try_custom_warmup(model, sample1)
+    warm_up_fn(
+        model,
+        sample1,
+        model_device,
+        transfer_to_device_fn,
+        num_runs=1,
+        batch_size=1,
+    )
 
     flops = measure_flops(model, sample1, print_details)
     if _is_valid(flops):
         results["flops"] = flops
-        logger.info(f"Model FLOPs: {flops} ({format_num(flops)})")
-
-    # Measure Allocated Memory
-    if model_device.type == "cuda":
-        pre_mem, post_mem, max_mem = measure_allocated_memory(
-            model, sample, model_device, transfer_to_device_fn, print_details
-        )
-        results["pre_inference_memory"] = pre_mem
-        results["max_inference_memory"] = max_mem
-        results["post_inference_memory"] = post_mem
-        logger.info(
-            f"Allocated GPU memory prior to inference: {pre_mem} ({format_num(pre_mem, bytes=True)})"
-        )
-        logger.info(
-            f"Allocated GPU memory after to inference: {post_mem} ({format_num(post_mem, bytes=True)})"
-        )
-        logger.info(
-            f"Max allocated GPU memory during inference: {max_mem} ({format_num(max_mem, bytes=True)})"
-        )
-    else:
-        logger.warning(
-            "Measurement of allocated memory is only available on CUDA devices"
-        )
+        print_fn(f"Model FLOPs: {flops} ({format_num(flops)})")
 
     # Measure inference timing
+    memory = {}
     timing = {}
     energy = {}
-    for bs in sorted(set([1, batch_size])):
-        s = sample1 if bs == 1 else sample
+    with torch.no_grad():
+        for bs in sorted(set([1, batch_size])):
+            s = sample1 if bs == 1 else sample
 
-        # Inference timing
-        warm_up(
-            model,
-            s,
-            model_device,
-            transfer_to_device_fn,
-            num_runs=max(1, num_runs // 10),
-            batch_size=batch_size,
-        )
-        if print_details:
-            measure_detailed_inference_timing(model, s, model_device)
+            # Measure Allocated Memory
+            if model_device.type == "cuda":
+                pre_mem, post_mem, max_mem = measure_allocated_memory(
+                    model, s, model_device, transfer_to_device_fn, print_details
+                )
+                memory[f"batch_size_{bs}"] = {
+                    "pre_inference_bytes": pre_mem,
+                    "max_inference_bytes": max_mem,
+                    "post_inference_bytes": post_mem,
+                    "pre_inference": format_num(pre_mem, bytes=True),
+                    "max_inference": format_num(max_mem, bytes=True),
+                    "post_inference": format_num(post_mem, bytes=True),
+                }
+                print_fn(
+                    fmt(
+                        {
+                            f"Memory results (batch_size={bs})": memory[
+                                f"batch_size_{bs}"
+                            ]
+                        }
+                    )
+                )
+            else:
+                logger.warning(
+                    "Measurement of allocated memory is only available on CUDA devices"
+                )
 
-        timing[f"batch_size_{bs}"] = measure_repeated_inference_timing(
-            model,
-            s,
-            model_device,
-            transfer_to_device_fn,
-            num_runs,
-            batch_size,
-        )
-        logger.info(
-            fmt({f"Timing results (batch_size={bs})": timing[f"batch_size_{bs}"]})
-        )
+            # Inference timing
+            warm_up_fn(
+                model,
+                s,
+                model_device,
+                transfer_to_device_fn,
+                num_runs=max(1, num_runs // 10),
+                batch_size=bs,
+            )
+            if print_details:
+                measure_detailed_inference_timing(model, s, model_device)
 
-        # Energy measurement
-        energy_joules = measure_energy(
-            model, sample, model_device, transfer_to_device_fn, print_details
-        )
-        if _is_valid(energy_joules):
-            energy_kwh = energy_joules / 3.6e6
-            energy[f"batch_size_{bs}"] = {
-                "joules": energy_joules,
-                "kWh": energy_kwh,
-            }
-            logger.info(f"Inference energy: {energy_joules} J ({energy_kwh} kWh)")
+            timing[f"batch_size_{bs}"] = measure_repeated_inference_timing(
+                model,
+                s,
+                model_device,
+                transfer_to_device_fn,
+                num_runs,
+                bs,
+            )
+            print_fn(
+                fmt({f"Timing results (batch_size={bs})": timing[f"batch_size_{bs}"]})
+            )
+
+            # Energy measurement
+            energy_joules = measure_energy(
+                model,
+                s,
+                model_device,
+                transfer_to_device_fn,
+                num_runs=max(1, num_runs // 10),
+                batch_size=bs,
+                include_transfer_costs=True,
+                print_fn=print_fn,
+            )
+            if _is_valid(energy_joules):
+                energy_kwh = energy_joules / 3.6e6
+                energy[f"batch_size_{bs}"] = {
+                    "joules": energy_joules,
+                    "kWh": energy_kwh,
+                }
+                print_fn(
+                    fmt(
+                        {
+                            f"Energy results (batch_size={bs})": energy[
+                                f"batch_size_{bs}"
+                            ]
+                        }
+                    )
+                )
 
     results["timing"] = timing
     if energy:
